import attr
import sqlalchemy as sa

@attr.s(frozen=True)
class SchemaTable(object):
  table: str = attr.ib()
  schema = attr.ib(default=None, validator=attr.validators.instance_of((str, type(None))))
  schema_table: str = attr.ib()
  stab: str = attr.ib() # 🗡
  st: str = attr.ib()
  db_url: str = attr.ib(default='sqlite:///:memory:') # in-memory db
  engine: sa.engine.base.Engine = attr.ib()

  @db_url.validator
  def check_url(self, attribute, value):
    pass

  @schema_table.default
  def schema_table_default(self):
    return self.table if self.schema is None else self.schema + '.' + self.table

  @stab.default # alias of schema_table
  def stab_default(self):
    return self.schema_table

  @st.default # even shorter alias of schema_table
  def st_default(self):
    return self.schema_table

  @engine.default
  def engine_default(self):
    return sa.create_engine(self.db_url)

  def todict(self):
    return attr.asdict(self) # TODO: exclude engine attr




from distutils.core import setup
setup(
  name = 'schematable', # How you named your package folder (MyLib)
  packages = ['schematable'], # Chose the same as "name"
  version = '0.2.0', # Start with a small number and increase it with every change you make
  license= 'MIT', # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'a Python utility library for working with SQL tables', # Give a short description about your library
  author = 'Jason Yung',
  author_email = 'json.yung@gmail.com',
  url = 'https://github.com/json2d/schematable', # Provide either the link to your github or to your website
  download_url = 'https://github.com/json2d/schematable/archive/v0.2.0.tar.gz',
  keywords = ['sql', 'schema', 'table', 'db', 'sqlalchemy'], # Keywords that define your package best
  install_requires= ['attrs'],
  classifiers=[
    'Development Status :: 3 - Alpha', # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers', # Define your audience
    'Topic :: Utilities',
    'License :: OSI Approved :: MIT License', # Again, pick a license
    'Programming Language :: Python :: 3', # Specify which pyhton versions that you want to support
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
  ],
)